import { HTMLAttributes, useCallback, useMemo } from 'react';
import { FC } from 'react';
import { DisplayConfig, LedStripConfigOfBorders } from '../models/display-config';
import { LedStrip } from './led-strip';
import tw, { css, styled, theme } from 'twin.macro';
import { ScreenshotDto } from '../models/screenshot.dto';
import { LedStripEditor } from './led-strip-editor';
import { LedStripConfig } from '../models/led-strip-config';
import debug from 'debug';
import { lensPath, lensProp, set, view } from 'ramda';

const logger = debug('app:display-with-led-strips');

export interface DisplayWithLedStripsProps
  extends Omit<HTMLAttributes<HTMLElement>, 'onChange'> {
  config: DisplayConfig;
  screenshot: ScreenshotDto;
  onChange?: (config: DisplayConfig) => void;
}

const StyledContainer = styled.section(
  tw`m-4 grid gap-1`,
  css`
    grid-template-columns: ${theme`width.5`} ${theme`width.3`} auto ${theme`width.3`} ${theme`width.5`};
  `,
  css`
    grid-template-rows: ${theme`width.5`} ${theme`width.3`} auto ${theme`width.3`} ${theme`width.5`};
  `,
);

export const DisplayWithLedStrips: FC<DisplayWithLedStripsProps> = ({
  config,
  screenshot,
  onChange,
  ...htmlAttrs
}) => {
  const screenshotUrl = useMemo(
    () => `data:image/ico;base64,${screenshot.encode_image}`,
    [screenshot.encode_image],
  );

  const onLedStripConfigChange = useCallback(
    (position: keyof LedStripConfigOfBorders, value: LedStripConfig | null) => {
      const xLens = lensPath<
        DisplayConfig,
        'led_strip_of_borders',
        keyof LedStripConfigOfBorders
      >(['led_strip_of_borders', position]);
      const c = set(xLens, value, config);
      logger('on change. prev: %o, curr: %o', view(xLens, config), value);
      onChange?.(c);
    },
    [config],
  );
  return (
    <StyledContainer {...htmlAttrs}>
      <img src={screenshotUrl} tw="row-start-3 col-start-3 w-full" />
      <LedStrip
        config={config.led_strip_of_borders.top}
        colors={screenshot.colors.top}
        tw="row-start-2 col-start-3"
      />
      <LedStrip
        config={config.led_strip_of_borders.left}
        colors={screenshot.colors.left}
        tw="row-start-3 col-start-2"
      />
      <LedStrip
        config={config.led_strip_of_borders.right}
        colors={screenshot.colors.right}
        tw="row-start-3 col-start-4"
      />
      <LedStrip
        config={config.led_strip_of_borders.bottom}
        colors={screenshot.colors.bottom}
        tw="row-start-4 col-start-3"
      />
      <LedStripEditor
        config={config.led_strip_of_borders.top}
        tw="row-start-1 col-start-3"
        onChange={(value) => onLedStripConfigChange('top', value)}
      />
      <LedStripEditor
        config={config.led_strip_of_borders.left}
        tw="row-start-3 col-start-1"
        onChange={(value) => onLedStripConfigChange('left', value)}
      />
      <LedStripEditor
        config={config.led_strip_of_borders.right}
        tw="row-start-3 col-start-5"
        onChange={(value) => onLedStripConfigChange('right', value)}
      />
      <LedStripEditor
        config={config.led_strip_of_borders.bottom}
        tw="row-start-5 col-start-3"
        onChange={(value) => onLedStripConfigChange('bottom', value)}
      />
    </StyledContainer>
  );
};
